@@ -9,7 +9,7 @@
  */
 /**
  * SECTION:x509vfy
- * @Short_description: X509 certificates verification support functions for Microsoft Cryptography API: Next Generation (CNG). 
+ * @Short_description: X509 certificates verification support functions for Microsoft Cryptography API: Next Generation (CNG).
  * @Stability: Private
  *
  */
@@ -23,17 +23,19 @@
 #include <windows.h>
 
 #include <xmlsec/xmlsec.h>
-#include <xmlsec/xmltree.h>
-#include <xmlsec/keys.h>
-#include <xmlsec/keyinfo.h>
-#include <xmlsec/keysmngr.h>
 #include <xmlsec/base64.h>
 #include <xmlsec/bn.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/keys.h>
+#include <xmlsec/keyinfo.h>
+#include <xmlsec/keysmngr.h>
+#include <xmlsec/xmltree.h>
 
 #include <xmlsec/mscng/crypto.h>
 #include <xmlsec/mscng/x509.h>
 
+#include "../cast_helpers.h"
+
 typedef struct _xmlSecMSCngX509StoreCtx xmlSecMSCngX509StoreCtx,
                                        *xmlSecMSCngX509StoreCtxPtr;
 struct _xmlSecMSCngX509StoreCtx {
@@ -43,11 +45,12 @@ struct _xmlSecMSCngX509StoreCtx {
     HCERTSTORE untrustedMemStore;
 };
 
-#define xmlSecMSCngX509StoreGetCtx(store) \
-    ((xmlSecMSCngX509StoreCtxPtr)(((xmlSecByte*)(store)) + \
-                 sizeof(xmlSecKeyDataStoreKlass)))
-#define xmlSecMSCngX509StoreSize \
-    (sizeof(xmlSecKeyDataStoreKlass) + sizeof(xmlSecMSCngX509StoreCtx))
+XMLSEC_KEY_DATA_STORE_DECLARE(MSCngX509Store, xmlSecMSCngX509StoreCtx)
+#define xmlSecMSCngX509StoreSize XMLSEC_KEY_DATA_STORE_SIZE(MSCngX509Store)
+
+static PCCERT_CONTEXT xmlSecMSCngX509FindCertByIssuerNameAndSerial        (HCERTSTORE store,
+                                                                           const xmlChar* issuerName,
+                                                                           const xmlChar* issuerSerial);
 
 static void
 xmlSecMSCngX509StoreFinalize(xmlSecKeyDataStorePtr store) {
@@ -112,7 +115,7 @@ xmlSecMSCngX509StoreAdoptKeyStore(xmlSecKeyDataStorePtr store, HCERTSTORE keySto
 
     ret = CertAddStoreToCollection(ctx->trusted, keyStore, CERT_PHYSICAL_STORE_ADD_ENABLE_FLAG, 2);
     if(ret != TRUE) {
-	xmlSecMSCngLastError("CertAddStoreToCollection",
+    xmlSecMSCngLastError("CertAddStoreToCollection",
             xmlSecKeyDataStoreGetName(store));
         return(-1);
     }
@@ -682,8 +685,8 @@ xmlSecMSCngUnixTimeToFileTime(time_t in, LPFILETIME out) {
     /* seconds -> 100 nanoseconds */
     /* 1970-01-01 epoch -> 1601-01-01 epoch */
     ll = Int32x32To64(in, 10000000) + 116444736000000000;
-    out->dwLowDateTime = (DWORD)ll;
-    out->dwHighDateTime = ll >> 32;
+    out->dwLowDateTime  = (DWORD)ll;
+    out->dwHighDateTime = (DWORD)(ll >> 32);
 
     return(0);
 }
@@ -864,114 +867,173 @@ xmlSecMSCngCertStrToName(DWORD dwCertEncodingType, LPTSTR pszX500, DWORD dwStrTy
 }
 
 static PCCERT_CONTEXT
-xmlSecMSCngX509FindCertByIssuer(HCERTSTORE store, LPTSTR wcIssuer,
-        xmlSecBnPtr issuerSerialBn, DWORD dwCertEncodingType) {
-    xmlSecAssert2(store != NULL, NULL);
-    xmlSecAssert2(wcIssuer != NULL, NULL);
-    xmlSecAssert2(issuerSerialBn != NULL, NULL);
-
+xmlSecMSCngX509FindCertByIssuerNameAndSerial(HCERTSTORE store, const xmlChar* issuerName, const xmlChar* issuerSerial) {
     PCCERT_CONTEXT res = NULL;
+    xmlSecBn issuerSerialBn;
+    int issuerSerialBnInitialized = 0;
+    LPTSTR wcIssuerName = NULL;
+    DWORD dwCertEncodingType = X509_ASN_ENCODING | PKCS_7_ASN_ENCODING;
     CERT_INFO certInfo;
-    BYTE* bdata;
+    BYTE* bdata = NULL;
+    xmlSecSize issuerSerialSize;
     DWORD len;
+    int ret;
 
+    xmlSecAssert2(store != 0, NULL);
+    xmlSecAssert2(issuerName != NULL, NULL);
+    xmlSecAssert2(issuerSerial != NULL, NULL);
 
-    xmlSecAssert2(store != NULL, NULL);
-    xmlSecAssert2(wcIssuer != NULL, NULL);
-    xmlSecAssert2(issuerSerialBn != NULL, NULL);
+    ret = xmlSecBnInitialize(&issuerSerialBn, 0);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBnInitialize", NULL);
+        goto done;
+    }
+    issuerSerialBnInitialized = 1;
 
-    certInfo.SerialNumber.cbData = xmlSecBnGetSize(issuerSerialBn);
-    certInfo.SerialNumber.pbData = xmlSecBnGetData(issuerSerialBn);
+    ret = xmlSecBnFromDecString(&issuerSerialBn, issuerSerial);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBnFromDecString", NULL);
+        goto done;
+    }
 
+    /* MS Windows wants this in the opposite order */
+    ret = xmlSecBnReverse(&issuerSerialBn);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecBnReverse", NULL);
+        goto done;
+    }
+
+    certInfo.SerialNumber.pbData = xmlSecBnGetData(&issuerSerialBn);
+    issuerSerialSize  = xmlSecBnGetSize(&issuerSerialBn);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(issuerSerialSize, certInfo.SerialNumber.cbData, goto done, NULL);
+
+    wcIssuerName = xmlSecMSCngX509GetCertName(issuerName);
+    if (wcIssuerName == NULL) {
+        xmlSecInternalError("xmlSecMSCngX509GetCertName", NULL);
+        goto done;
+    }
 
     /* CASE 1: UTF8, DN */
     if (NULL == res) {
         bdata = xmlSecMSCngCertStrToName(dwCertEncodingType,
-                    wcIssuer,
-                    CERT_NAME_STR_ENABLE_UTF8_UNICODE_FLAG | CERT_OID_NAME_STR,
-                    &len);
-        if(bdata != NULL) {
+            wcIssuerName,
+            CERT_NAME_STR_ENABLE_UTF8_UNICODE_FLAG | CERT_OID_NAME_STR,
+            &len);
+        if (bdata != NULL) {
             certInfo.Issuer.cbData = len;
             certInfo.Issuer.pbData = bdata;
 
             res = CertFindCertificateInStore(store,
-                        dwCertEncodingType,
-                        0,
-                        CERT_FIND_SUBJECT_CERT,
-                        &certInfo,
-                        NULL);
+                dwCertEncodingType,
+                0,
+                CERT_FIND_SUBJECT_CERT,
+                &certInfo,
+                NULL);
             xmlFree(bdata);
+            bdata = NULL;
         }
     }
 
     /* CASE 2: UTF8, REVERSE DN */
     if (NULL == res) {
         bdata = xmlSecMSCngCertStrToName(dwCertEncodingType,
-                    wcIssuer,
-                    CERT_NAME_STR_ENABLE_UTF8_UNICODE_FLAG | CERT_OID_NAME_STR | CERT_NAME_STR_REVERSE_FLAG,
-                    &len);
-        if(bdata != NULL) {
+            wcIssuerName,
+            CERT_NAME_STR_ENABLE_UTF8_UNICODE_FLAG | CERT_OID_NAME_STR | CERT_NAME_STR_REVERSE_FLAG,
+            &len);
+        if (bdata != NULL) {
             certInfo.Issuer.cbData = len;
             certInfo.Issuer.pbData = bdata;
 
             res = CertFindCertificateInStore(store,
-                        dwCertEncodingType,
-                        0,
-                        CERT_FIND_SUBJECT_CERT,
-                        &certInfo,
-                        NULL);
+                dwCertEncodingType,
+                0,
+                CERT_FIND_SUBJECT_CERT,
+                &certInfo,
+                NULL);
             xmlFree(bdata);
+            bdata = NULL;
         }
     }
 
     /* CASE 3: UNICODE, DN */
     if (NULL == res) {
         bdata = xmlSecMSCngCertStrToName(dwCertEncodingType,
-                    wcIssuer,
-                    CERT_OID_NAME_STR,
-                    &len);
-        if(bdata != NULL) {
+            wcIssuerName,
+            CERT_OID_NAME_STR,
+            &len);
+        if (bdata != NULL) {
             certInfo.Issuer.cbData = len;
             certInfo.Issuer.pbData = bdata;
 
             res = CertFindCertificateInStore(store,
-                        dwCertEncodingType,
-                        0,
-                        CERT_FIND_SUBJECT_CERT,
-                        &certInfo,
-                        NULL);
+                dwCertEncodingType,
+                0,
+                CERT_FIND_SUBJECT_CERT,
+                &certInfo,
+                NULL);
             xmlFree(bdata);
+            bdata = NULL;
         }
     }
 
     /* CASE 4: UNICODE, REVERSE DN */
     if (NULL == res) {
         bdata = xmlSecMSCngCertStrToName(dwCertEncodingType,
-                    wcIssuer,
-                    CERT_OID_NAME_STR | CERT_NAME_STR_REVERSE_FLAG,
-                    &len);
-        if(bdata != NULL) {
+            wcIssuerName,
+            CERT_OID_NAME_STR | CERT_NAME_STR_REVERSE_FLAG,
+            &len);
+        if (bdata != NULL) {
             certInfo.Issuer.cbData = len;
             certInfo.Issuer.pbData = bdata;
 
             res = CertFindCertificateInStore(store,
-                        dwCertEncodingType,
-                        0,
-                        CERT_FIND_SUBJECT_CERT,
-                        &certInfo,
-                        NULL);
+                dwCertEncodingType,
+                0,
+                CERT_FIND_SUBJECT_CERT,
+                &certInfo,
+                NULL);
             xmlFree(bdata);
+            bdata = NULL;
         }
     }
 
-    return (res);
+done:
+    if (bdata != NULL) {
+        xmlFree(bdata);
+    }
+    if (wcIssuerName != NULL) {
+        xmlFree(wcIssuerName);
+    }
+    if (issuerSerialBnInitialized) {
+        xmlSecBnFinalize(&issuerSerialBn);
+    }
+    return(res);
+}
+
+static PCCERT_CONTEXT
+xmlSecMSCngX509FindCertBySki(HCERTSTORE store, xmlSecByte* ski, xmlSecSize skiSize) {
+    CRYPT_HASH_BLOB blob;
+
+    xmlSecAssert2(store != 0, NULL);
+    xmlSecAssert2(ski != NULL, NULL);
+    xmlSecAssert2(skiSize > 0, NULL);
+
+    blob.pbData = ski;
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(skiSize, blob.cbData, return(NULL), NULL);
+
+    return(CertFindCertificateInStore(store,
+        PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
+        0,
+        CERT_FIND_KEY_IDENTIFIER,
+        &blob,
+        NULL));
 }
 
 static PCCERT_CONTEXT
 xmlSecMSCngX509FindCert(HCERTSTORE store, xmlChar* subjectName,
-        xmlChar* issuerName, xmlChar* issuerSerial, xmlChar* ski) {
-    PCCERT_CONTEXT cert;
-    int ret;
+                        xmlChar* issuerName, xmlChar* issuerSerial,
+                        xmlSecByte* ski, xmlSecSize skiSize) {
+    PCCERT_CONTEXT cert = NULL;
 
     xmlSecAssert2(store != 0, NULL);
 
@@ -987,83 +1049,17 @@ xmlSecMSCngX509FindCert(HCERTSTORE store, xmlChar* subjectName,
         cert = xmlSecMSCngX509FindCertBySubject(store, wcSubjectName,
             PKCS_7_ASN_ENCODING | X509_ASN_ENCODING);
         xmlFree(wcSubjectName);
-
-        return(cert);
     }
 
     if(issuerName != NULL && issuerSerial != NULL) {
-        xmlSecBn issuerSerialBn;
-        LPTSTR wcIssuerName = NULL;
-
-        ret = xmlSecBnInitialize(&issuerSerialBn, 0);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecBnInitialize", NULL);
-            return(NULL);
-        }
-
-        ret = xmlSecBnFromDecString(&issuerSerialBn, issuerSerial);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecBnFromDecString", NULL);
-            xmlSecBnFinalize(&issuerSerialBn);
-            return(NULL);
-        }
-
-        /* xmlSecMSCngX509FindCertByIssuer() wants this in the opposite order */
-        ret = xmlSecBnReverse(&issuerSerialBn);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecBnReverse", NULL);
-            xmlSecBnFinalize(&issuerSerialBn);
-            return(NULL);
-        }
-
-        wcIssuerName = xmlSecMSCngX509GetCertName(issuerName);
-        if(wcIssuerName == NULL) {
-            xmlSecInternalError("xmlSecMSCngX509GetCertName", NULL);
-            xmlSecBnFinalize(&issuerSerialBn);
-            return(NULL);
-        }
-
-        cert = xmlSecMSCngX509FindCertByIssuer(store, wcIssuerName,
-            &issuerSerialBn, X509_ASN_ENCODING | PKCS_7_ASN_ENCODING);
-        xmlFree(wcIssuerName);
-        xmlSecBnFinalize(&issuerSerialBn);
-
-        return(cert);
+        cert = xmlSecMSCngX509FindCertByIssuerNameAndSerial(store, issuerName, issuerSerial);
     }
 
-    if(ski != NULL) {
-        CRYPT_HASH_BLOB blob;
-        xmlChar* binSki;
-        int binSkiLen;
-
-        binSki = xmlStrdup(ski);
-        if(binSki == NULL) {
-            xmlSecStrdupError(ski, NULL);
-            return (NULL);
-        }
-
-        /* base64 decode "in place" */
-        binSkiLen = xmlSecBase64Decode(binSki, (xmlSecByte*)binSki, xmlStrlen(binSki));
-        if(binSkiLen < 0) {
-            xmlSecInternalError("xmlSecBase64Decode", NULL);
-            xmlFree(binSki);
-            return(NULL);
-        }
-
-        blob.pbData = binSki;
-        blob.cbData = binSkiLen;
-        cert = CertFindCertificateInStore(store,
-                        PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
-                        0,
-                        CERT_FIND_KEY_IDENTIFIER,
-                        &blob,
-                        NULL);
-        xmlFree(binSki);
-
-	return(cert);
+    if((ski != NULL) && (skiSize > 0)) {
+        cert = xmlSecMSCngX509FindCertBySki(store, ski, skiSize);
     }
 
-    return(NULL);
+    return(cert);
 }
 
 /**
@@ -1082,27 +1078,68 @@ xmlSecMSCngX509FindCert(HCERTSTORE store, xmlChar* subjectName,
  */
 PCCERT_CONTEXT
 xmlSecMSCngX509StoreFindCert(xmlSecKeyDataStorePtr store, xmlChar *subjectName,
-    xmlChar *issuerName, xmlChar *issuerSerial, xmlChar *ski,
-    xmlSecKeyInfoCtx* keyInfoCtx) {
+                            xmlChar* issuerName, xmlChar* issuerSerial, xmlChar* ski,
+                            xmlSecKeyInfoCtx* keyInfoCtx) {
+    if (ski != NULL) {
+        xmlSecSize skiDecodedSize = 0;
+        int ret;
+
+        /* our usual trick with base64 decode */
+        ret = xmlSecBase64DecodeInPlace(ski, &skiDecodedSize);
+        if (ret < 0) {
+            xmlSecInternalError2("xmlSecBase64DecodeInPlace", NULL,
+                "ski=%s", xmlSecErrorsSafeString(ski));
+            return(NULL);
+        }
+
+        return(xmlSecMSCngX509StoreFindCert_ex(store, subjectName, issuerName, issuerSerial,
+            (xmlSecByte*)ski, skiDecodedSize, keyInfoCtx));
+    } else {
+        return(xmlSecMSCngX509StoreFindCert_ex(store, subjectName, issuerName, issuerSerial,
+            NULL, 0, keyInfoCtx));
+
+    }
+}
+
+/**
+ * xmlSecMSCngX509StoreFindCert_ex:
+ * @store:          the pointer to X509 key data store klass.
+ * @subjectName:    the desired certificate name.
+ * @issuerName:     the desired certificate issuer name.
+ * @issuerSerial:   the desired certificate issuer serial number.
+ * @ski:            the desired certificate SKI.
+ * @skiSize:        the desired certificate SKI size.
+ * @keyInfoCtx:     the pointer to <dsig:KeyInfo/> element processing context.
+ *
+ * Searches @store for a certificate that matches given criteria.
+ *
+ * Returns: pointer to found certificate or NULL if certificate is not found
+ * or an error occurs.
+ */
+PCCERT_CONTEXT
+xmlSecMSCngX509StoreFindCert_ex(xmlSecKeyDataStorePtr store, xmlChar* subjectName,
+                                xmlChar* issuerName, xmlChar* issuerSerial,
+                                xmlSecByte* ski, xmlSecSize skiSize,
+                                xmlSecKeyInfoCtx* keyInfoCtx ATTRIBUTE_UNUSED) {
     xmlSecMSCngX509StoreCtxPtr ctx;
     PCCERT_CONTEXT cert = NULL;
 
     xmlSecAssert2(xmlSecKeyDataStoreCheckId(store, xmlSecMSCngX509StoreId), NULL);
-    xmlSecAssert2(keyInfoCtx != NULL, NULL);
+    UNREFERENCED_PARAMETER(keyInfoCtx);
 
     ctx = xmlSecMSCngX509StoreGetCtx(store);
     xmlSecAssert2(ctx != NULL, NULL);
 
     /* search untrusted certs store */
-    if(ctx->untrusted != NULL) {
+    if (ctx->untrusted != NULL) {
         cert = xmlSecMSCngX509FindCert(ctx->untrusted, subjectName,
-            issuerName, issuerSerial, ski);
+            issuerName, issuerSerial, ski, skiSize);
     }
 
     /* search trusted certs store */
-    if(cert == NULL && ctx->trusted != NULL) {
+    if (cert == NULL && ctx->trusted != NULL) {
         cert = xmlSecMSCngX509FindCert(ctx->trusted, subjectName,
-            issuerName, issuerSerial, ski);
+            issuerName, issuerSerial, ski, skiSize);
     }
 
     return(cert);
